@@ -90,10 +90,11 @@ func init() {
 		- http://ascii-table.com/ansi-escape-sequences.php
 		- http://ascii-table.com/ansi-escape-sequences-vt-100.php
 		- http://tldp.org/HOWTO/Bash-Prompt-HOWTO/x405.html
+		- https://invisible-island.net/xterm/ctlseqs/ctlseqs.html
 	*/
 	// The following regular expression will include not all but most of the
 	// frequently used ANSI sequences
-	ansiRegex = regexp.MustCompile("(?:\x1b[\\[()][0-9;]*[a-zA-Z@]|\x1b.|[\x0e\x0f]|.\x08)")
+	ansiRegex = regexp.MustCompile("(?:\x1b[\\[()][0-9;]*[a-zA-Z@]|\x1b][0-9];[[:print:]]+(?:\x1b\\\\|\x07)|\x1b.|[\x0e\x0f]|.\x08)")
 }
 
 func findAnsiStart(str string) int {
@@ -243,6 +244,10 @@ func interpretCode(ansiCode string, prevState *ansiState) *ansiState {
 					state.attr = state.attr | tui.Blink
 				case 7:
 					state.attr = state.attr | tui.Reverse
+				case 23: // tput rmso
+					state.attr = state.attr &^ tui.Italic
+				case 24: // tput rmul
+					state.attr = state.attr &^ tui.Underline
 				case 0:
 					init()
 				default:
